# This file was auto-generated by Fern from our API Definition.

import typing

from .biomarker_result import BiomarkerResult

LabResultsRawResults = typing.Union[typing.List[BiomarkerResult], typing.Dict[str, typing.Any]]
